/*
This is a part of the fonts.dll LiteStep module source code.

Copyright (C) 2003 ilmcuts

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Includes
//
#include "../ls-b247/lsapi/lsapi.h"
#include "fonts.h"
#include <vector>
#include <string>


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// globals/typedefs
//
typedef std::vector<std::string> FontVec;

FontVec g_Fonts;
HMODULE g_hGdi = NULL;
int (WINAPI *pAddFontResourceEx)(LPCSTR, DWORD, PVOID) = NULL;
int (WINAPI *pRemoveFontResourceEx)(LPCSTR, DWORD, PVOID) = NULL;

#ifndef FR_PRIVATE
# define FR_PRIVATE     0x10
#endif


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ErrorMessage
//
// Error handler
//
void ErrorMessage(LPCSTR pszMessage)
{
    MessageBox(NULL, pszMessage, "envvars error",
        MB_ICONERROR | MB_TOPMOST | MB_SETFOREGROUND);
}


void ErrorMessagef(LPCSTR pszFormat, ...)
{
    char szMessage[MAX_LINE_LENGTH];
    va_list argList;
    
    va_start(argList, pszFormat);
    vsprintf(szMessage, pszFormat, argList);
    va_end(argList);

    ErrorMessage(szMessage);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// LoadFont
//
// Parses input line and attempts to export variable
//
void LoadFont(LPCSTR pszFile)
{
    BOOL bSuccess = false;
    
    if (pAddFontResourceEx)
    {
        bSuccess = pAddFontResourceEx(pszFile, FR_PRIVATE, 0);
    }
    else
    {
        bSuccess = AddFontResource(pszFile);
    }
    
    if(bSuccess)
    {
        g_Fonts.push_back(pszFile);
    }
    else
    {
        ErrorMessagef("Failed to load font: %s", pszFile);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// initModuleEx
//
// Module Initialization
//
int initModuleEx(HWND /* hLiteStep */, HINSTANCE /* hInstance */,
                 LPCSTR /* szLSpath */)
{
    g_hGdi = LoadLibrary("GDI32.DLL");

    pAddFontResourceEx = (int (WINAPI*)(LPCSTR, DWORD, PVOID))
        GetProcAddress(g_hGdi, "AddFontResourceExA");

    if (pAddFontResourceEx)
    {
        pRemoveFontResourceEx = (int (WINAPI*)(LPCSTR, DWORD, PVOID))
            GetProcAddress(g_hGdi, "RemoveFontResourceExA");

        if (!pRemoveFontResourceEx)
        {
            pAddFontResourceEx = NULL;
        }
    }

    FILE* fpStep = LCOpen(NULL);
    
    if (fpStep)
    {
        char szLineBuffer[MAX_LINE_LENGTH] = { 0 };
        
        while (LCReadNextConfig(fpStep, "*Loadfont", szLineBuffer, MAX_LINE_LENGTH))
        {
            char szFile[MAX_LINE_LENGTH] = { 0 };
            char* ppszTokens[] = { NULL, szFile };
            
            LCTokenize(szLineBuffer, ppszTokens, 2, NULL);
            LoadFont(szFile);
        }
        
        LCClose(fpStep);
    }

    return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// quitModule
//
// module cleanup
//
int quitModule(HINSTANCE /* hInstance */)
{
    for (FontVec::iterator iter = g_Fonts.begin(); iter != g_Fonts.end(); ++iter)
    {
        if (pRemoveFontResourceEx)
        {
            pRemoveFontResourceEx(iter->c_str(), FR_PRIVATE, 0);
        }
        else
        {
            RemoveFontResource(iter->c_str());
        }
    }

    g_Fonts.clear();

    FreeLibrary(g_hGdi);

    return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DllMain
//
// DLL entry point
//
BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason,
                      LPVOID /* pvReserved */)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        DisableThreadLibraryCalls(hInstance);
    }
    
    return TRUE;
}
